%% Parameter Sweep of ODEs
% This is a parameter sweep study of a 2nd order ODE system.
%
%%
%
%   m*X'' + b*X' + k*X = 0
%
% We solve the ODE for a time span of 0 to 25 seconds, with initial
% conditions X(0) = 0 and X'(0) = 1. We sweep the parameters "b" and "k"
% and record the peak values of X for each condition. At the end, we plot a
% surface of the results.
%
% In this version of the implementation matlabpool and parfor are used to
% distribute the load
%
% Copyright 2009 The MathWorks, Inc.
% maxNumCompThreads(4)

%% Initialize Problem
m     =         5;  % mass
bVals = 0.1:.025:5;   % damping values (step .05)
kVals = 1.5:.05:5;   % stiffness values (step .1)damping
[kGrid, bGrid] = meshgrid(bVals, kVals);
peakVals = nan(size(kGrid));

try
    matlabpool('addattachedfiles', {'odesystem.m'})
catch ME
    disp('File was already attached.')
end


%% Parameter Sweep
disp('Computing ...');drawnow;

tic;

% Run serial for-loop or parallel parfor-loop, compare times

parfor idx = 1:numel(kGrid)   

  % Solve ODE
  [T,Y] = ode45(@(t,y) odesystem(t, y, m, bGrid(idx), kGrid(idx)), ...
    [0, 250], ...  % simulate for 250 seconds
    [0, 1]) ;      % initial conditions
  
  % Determine peak value
  peakVals(idx) = max(Y(:,1));
end

toc

%% Visualize

% figure;
%f=surf(bVals, kVals, peakVals);
%set(f,'LineStyle','none')
%set(f,'FaceAlpha',0.5)
%xlabel('Damping'); ylabel('Stiffness'); zlabel('Peak Displacement');
%view(50, 30)
% % set(gcf,'WindowStyle','docked')

%%
% print(gcf,'-dpng','paramSweep.png')
